require "helium/errors/version"

module Helium
  class Errors
    Failure = Struct.new(:key, :message, :options) do
      def inspect_message
        text = message.to_s
        text += " (#{options.map {|key, value| "#{key}: #{value}"}.join(", ")})" if options.any?
        text
      end

      def inspect
        "<Error: #{key} is #{inspect_message}>"
      end
    end

    def initialize
      @failures = {}
    end

    def add(key, message, **options)
      (@failures[key] ||= []) << Failure.new(key, message, options)
      self
    end

    def to_h
      @failures.transform_values do |failures|
        failures.map { |failure| [failure.message, failure.options] }
      end
    end

    def inspect
      text = "# Helium::Errors"
      return text + " {}" if @failures.none?

      @failures.each do |key, failures|

        text += "#{$/}|   #{key}: #{failures.map(&:inspect_message).join(", ")}"
      end
      text
    end
  end
end
